@@ -79,7 +79,6 @@ enum {
 	PROP_HIGHLIGHT_CURRENT_LINE
 };
 
-
 struct _GtkSourceViewPrivate
 {
 	guint		 tabs_width;
@@ -638,50 +637,37 @@ set_source_buffer (GtkSourceView *view, GtkTextBuffer *buffer)
 	}
 }
 
-static void
-scroll_to_cursor (GtkSourceView *view)
-{
-	GtkTextBuffer* buffer = NULL;
-	
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
-
-	gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view),
-				gtk_text_buffer_get_mark (buffer,
-				"insert"));
-}
-
 static void
 gtk_source_view_undo (GtkSourceView *view)
 {
-	GtkSourceBuffer *buffer;
-	
+	GtkTextBuffer *buffer;
+
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (view));
 
-	buffer = GTK_SOURCE_BUFFER (
-			gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-		
-	if (gtk_source_buffer_can_undo (buffer))
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	if (gtk_source_buffer_can_undo (GTK_SOURCE_BUFFER (buffer)))
 	{
-		gtk_source_buffer_undo (buffer);
-		scroll_to_cursor (view);
+		gtk_source_buffer_undo (GTK_SOURCE_BUFFER (buffer));
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view),
+						    gtk_text_buffer_get_insert (buffer));
 	}
 }
 
 static void
 gtk_source_view_redo (GtkSourceView *view)
 {
-	GtkSourceBuffer *buffer;
+	GtkTextBuffer *buffer;
 
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (view));
 
-	buffer = GTK_SOURCE_BUFFER (
-			gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-		
-	if (gtk_source_buffer_can_redo (buffer))
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	if (gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (buffer)))
 	{
-		gtk_source_buffer_redo (buffer);
-		scroll_to_cursor (view);
+		gtk_source_buffer_redo (GTK_SOURCE_BUFFER (buffer));
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view),
+						    gtk_text_buffer_get_insert (buffer));
 	}
 }
 
@@ -1269,7 +1255,7 @@ gtk_source_view_expose (GtkWidget      *widget,
 			gint y;
 			gint height;
 			gint win_y;
-
+			
 			gtk_text_buffer_get_iter_at_mark (text_view->buffer, 
 							  &cur, 
 							  gtk_text_buffer_get_insert (text_view->buffer));
@@ -1295,14 +1281,10 @@ gtk_source_view_expose (GtkWidget      *widget,
 			redraw_rect.width = visible_rect.width;
 			redraw_rect.height = visible_rect.height;
 
-			gtk_paint_flat_box (widget->style, 
-					    event->window, 
-					    GTK_WIDGET_STATE (widget),
-					    GTK_SHADOW_NONE,
-					    &redraw_rect, 
-					    widget,
-					    "current_line", 
-					    MAX (1, gtk_text_view_get_left_margin (text_view) - 1),
+			gdk_draw_rectangle (event->window,
+					    widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
+					    TRUE,
+					    redraw_rect.x + MAX (0, gtk_text_view_get_left_margin (text_view) - 1),
 					    win_y,
 					    redraw_rect.width,
 					    height);
@@ -1354,7 +1336,6 @@ gtk_source_view_expose (GtkWidget      *widget,
 	return event_handled;	
 }
 
-
 /*
  *This is a pretty important function...we call it when the tab_stop is changed,
  *And when the font is changed.
